@@ -106,6 +106,11 @@ class SFTDataConfig(BaseModel):
         description="Dataset configuration for SFT training. It includes dataset name, subset, revision, train split, and test split.",
     )
 
+    mini_batch: int = Field(
+        default=2,
+        description="mini-batch size for training.",
+    )
+
     dataloader_shuffle: bool = Field(
         default=False,
         description="Shuffle the dataloader. If False, the dataloader will be used in the order it is loaded.",
@@ -360,6 +365,8 @@ class SubProfilerConfig(BaseModel):
         default=False, description="Whether to profile, only used in runtime."
     )
     active_steps: int = Field(default=1, description="Number of active steps")
+    warmup_steps: int = Field(default=1, description="Number of warmup steps")
+    wait_steps: int = Field(default=1, description="Number of wait steps")
     rank_filter: List[int] = Field(default_factory=list, description="Rank filter")
     record_shape: bool = Field(default=False, description="Whether to record shape")
     profile_memory: bool = Field(default=False, description="Whether to profile memory")
@@ -671,11 +678,10 @@ class RolloutConfig(BaseModel):
         description="Batch size for rollout generation during validation.",
     )
 
-    # not used yet.
     quantization: str = Field(
         default="none",
         description="Quantization in vllm rollout generation.",
-        choices=["none"],
+        choices=["none", "fp8"],
     )
 
     seed: Optional[int] = Field(default=None, description="random seed for rollout.")
