@@ -24,7 +24,11 @@ from cosmos_rl.policy.config import (
 )
 from cosmos_rl.utils.util import compute_mfu
 from cosmos_rl.utils.logging import logger
-from cosmos_rl.utils.wandb_logger import is_wandb_available, log_wandb
+from cosmos_rl.utils.wandb_logger import (
+    init_wandb,
+    is_wandb_available,
+    log_wandb,
+)
 import torch
 import numpy as np
 from torch.utils.data import Dataset
@@ -40,6 +44,23 @@ import os
 from typing import Optional, Dict, Any
 from tqdm import tqdm
 from cosmos_rl.utils.ulysses import slice_inputs_for_ulysses
+from functools import partial
+
+
+def async_safe_ce(
+    output: torch.Tensor,
+    target: torch.LongTensor,
+    ignore_index: int = -100,
+    loss_scaling_factor: float = 1.0,
+) -> torch.Tensor:
+    loss = torch.nn.functional.cross_entropy(
+        output[:, :-1].flatten(0, 1),
+        target[:, 1:].flatten(0, 1),
+        ignore_index=ignore_index,
+        reduction="mean",
+    )
+    # In case of all labels are ignored, loss will be nan.
+    return torch.nan_to_num(loss, nan=0.0) * loss_scaling_factor
 
 
 def collate_fn(
@@ -225,6 +246,16 @@ class SFTTrainer(Trainer):
         if parallel_dims.dp_enabled:
             self.dp_rank = parallel_dims.mesh["dp"].get_local_rank()
             self.dp_world_size = parallel_dims.mesh["dp"].size()
+
+        # Prepare wandb
+        if "wandb" in config.logging.logger and is_wandb_available():
+            init_wandb(config, parallel_dims)
+        else:
+            logger.warning(
+                "Wandb is not available. Please install it to use wandb logging features."
+            )
+
+        # Prepare dataset
         train_dataset, val_dataset = construct_dataset(
             config.train.train_policy,
             tokenizer=self.tokenizer,
@@ -303,9 +334,9 @@ class SFTTrainer(Trainer):
                     )
                 else:
                     self.train_step = ckpt_extra_vars.get("step", 0)
-            except Exception:
+            except Exception as e:
                 logger.error(
-                    f"Cannot resume from {self.config.train.resume}. Trying to load from HuggingFace..."
+                    f"Cannot resume due to error: {e}. Trying to load from HuggingFace..."
                 )
                 self.model.load_hf_weights(
                     config.policy.model_name_or_path, parallel_dims, self.device
@@ -316,7 +347,7 @@ class SFTTrainer(Trainer):
             )
         self.model.train()
 
-        self.loss_fn = torch.nn.CrossEntropyLoss()
+        self.loss_fn = async_safe_ce
 
     def validate(self):
         logger.info(f"Validation at step {self.train_step}/{self.total_steps}...")
@@ -375,15 +406,11 @@ class SFTTrainer(Trainer):
                         )
 
                     if pp_last_stage:
-                        val_logits = pp_out[:, :-1].contiguous()
-                        val_loss = self.loss_fn(
-                            val_logits.view(-1, val_logits.size(-1)),
-                            val_labels[:, 1:].contiguous().view(-1),
-                        )
+                        val_loss = self.loss_fn(pp_out, val_labels)
                     else:
                         val_loss = torch.tensor([-1.0], device=self.device)
                 else:
-                    val_logits = self.model(**val_batch)[:, :-1].contiguous()
+                    val_logits = self.model(**val_batch)
 
                     # recover from ulysses if cp is enabled
                     if self.parallel_dims.cp_enabled:
@@ -392,10 +419,7 @@ class SFTTrainer(Trainer):
                         if padding_mask_before_cp is not None:
                             val_batch["padding_mask"] = padding_mask_before_cp
 
-                    val_loss = self.loss_fn(
-                        val_logits.view(-1, val_logits.size(-1)),
-                        val_labels[:, 1:].contiguous().view(-1),
-                    )
+                    val_loss = self.loss_fn(val_logits, val_labels)
                 val_total_loss += val_loss.item() * val_inputs.size(0)
             val_avg_loss = val_total_loss / len(self.val_data_loader.dataset)
             logger.info(f"Validation loss: {val_avg_loss}")
@@ -417,109 +441,125 @@ class SFTTrainer(Trainer):
 
         for cur_epoch in range(start_epoch, self.epoch):
             logger.info(f"Training epoch {cur_epoch + 1}/{self.epoch}")
-            for batch in self.train_data_loader:
+            for global_batch in self.train_data_loader:
                 if data_loader_bias > 0:
                     data_loader_bias -= 1
                     continue
 
-                # if [profiler.enable_nsys] is true, cudaProfilerStart() / cudaProfilerStop() are used to trigger nsys capture
-                # settings from [profiler.sub_profiler_config] are reused
-                if (
-                    self.config.profiler.enable_nsys
-                    and self.profiler.global_rank in self.profiler.rank_filter
-                ):
-                    if (
-                        self.train_step
-                        == self.profiler.WAIT_STEPS + self.profiler.WARMUP_STEPS
-                    ):
-                        torch.cuda.cudart().cudaProfilerStart()
-                    elif (
-                        self.train_step
-                        == self.profiler.WAIT_STEPS
-                        + self.profiler.WARMUP_STEPS
-                        + self.profiler.active_steps
-                    ):
-                        torch.cuda.cudart().cudaProfilerStop()
-
-                self.model.train()
-                start_event = torch.cuda.Event(enable_timing=True)
-                end_event = torch.cuda.Event(enable_timing=True)
-                start_event.record()
-                for k, v in batch.items():
-                    batch[k] = v.to(self.device) if isinstance(v, torch.Tensor) else v
-
-                labels = batch.pop("label_ids")
-
-                position_ids, input_ids, pos_seq_dim = self.model.get_position_ids(
-                    **batch
-                )
-
-                batch["position_ids"] = position_ids
-                padding_mask = batch.get("padding_mask", None)
-
-                if self.parallel_dims.cp_enabled:
-                    input_ids_before_cp = input_ids
-                    position_ids_before_cp = position_ids
-                    padding_mask_before_cp = padding_mask
-
-                    [input_ids, position_ids, padding_mask] = slice_inputs_for_ulysses(
-                        [input_ids, position_ids, padding_mask],
-                        self.parallel_dims.mesh["cp"],
+                global_batch_size = global_batch["input_ids"].shape[0]
+                # split global_batch into mini_batches
+                mini_batches = [
+                    {
+                        k: v[i : i + self.config.train.train_policy.mini_batch]
+                        for k, v in global_batch.items()
+                    }
+                    for i in range(
+                        0, global_batch_size, self.config.train.train_policy.mini_batch
                     )
+                ]
 
-                    batch["input_ids"] = input_ids
-                    batch["position_ids"] = position_ids
-                    if padding_mask is not None:
-                        batch["padding_mask"] = padding_mask
-
+                acc_loss = torch.zeros(1, device=self.device)
                 self.optimizers.zero_grad()
+                for batch in mini_batches:
+                    # if [profiler.enable_nsys] is true, cudaProfilerStart() / cudaProfilerStop() are used to trigger nsys capture
+                    # settings from [profiler.sub_profiler_config] are reused
+                    if (
+                        self.config.profiler.enable_nsys
+                        and self.profiler.global_rank in self.profiler.rank_filter
+                    ):
+                        if (
+                            self.train_step
+                            == self.profiler.wait_steps + self.profiler.warmup_steps
+                        ):
+                            torch.cuda.cudart().cudaProfilerStart()
+                        elif (
+                            self.train_step
+                            == self.profiler.wait_steps
+                            + self.profiler.warmup_steps
+                            + self.profiler.active_steps
+                        ):
+                            torch.cuda.cudart().cudaProfilerStop()
 
-                if self.parallel_dims.pp_enabled:
-                    pp_last_stage = (
-                        self.parallel_dims.pp_coord[0]
-                        == self.parallel_dims.pp_coord[1] - 1
+                    self.model.train()
+                    start_event = torch.cuda.Event(enable_timing=True)
+                    end_event = torch.cuda.Event(enable_timing=True)
+                    start_event.record()
+                    for k, v in batch.items():
+                        batch[k] = (
+                            v.to(self.device) if isinstance(v, torch.Tensor) else v
+                        )
+
+                    labels = batch.pop("label_ids")
+
+                    position_ids, input_ids, pos_seq_dim = self.model.get_position_ids(
+                        **batch
                     )
-                    pp_first_stage = self.parallel_dims.pp_coord[0] == 0
 
-                    # Pipeline Parallel forward / backward inside step() call
-                    targets, losses = (labels, []) if pp_last_stage else (None, None)
-                    if pp_first_stage:
-                        self.pp_scheduler.step(
-                            **batch,
-                            pp_dynamic_shape_enabled=self.parallel_dims.pp_dynamic_shape_enabled,
-                            seq_len_multiple=self.seq_len_multiple,
+                    batch["position_ids"] = position_ids
+                    padding_mask = batch.get("padding_mask", None)
+
+                    if self.parallel_dims.cp_enabled:
+                        input_ids_before_cp = input_ids
+                        position_ids_before_cp = position_ids
+                        padding_mask_before_cp = padding_mask
+
+                        [input_ids, position_ids, padding_mask] = (
+                            slice_inputs_for_ulysses(
+                                [input_ids, position_ids, padding_mask],
+                                self.parallel_dims.mesh["cp"],
+                            )
+                        )
+
+                        batch["input_ids"] = input_ids
+                        batch["position_ids"] = position_ids
+                        if padding_mask is not None:
+                            batch["padding_mask"] = padding_mask
+
+                    if self.parallel_dims.pp_enabled:
+                        pp_last_stage = (
+                            self.parallel_dims.pp_coord[0]
+                            == self.parallel_dims.pp_coord[1] - 1
+                        )
+                        pp_first_stage = self.parallel_dims.pp_coord[0] == 0
+
+                        # Pipeline Parallel forward / backward inside step() call
+                        targets, losses = (
+                            (labels, []) if pp_last_stage else (None, None)
+                        )
+                        if pp_first_stage:
+                            self.pp_scheduler.step(
+                                **batch,
+                                pp_dynamic_shape_enabled=self.parallel_dims.pp_dynamic_shape_enabled,
+                                seq_len_multiple=self.seq_len_multiple,
+                            )
+                        else:
+                            # FWD + BWD if it is 1F1B-like scheduler
+                            self.pp_scheduler.step(
+                                position_ids=batch["position_ids"],
+                                target=targets,
+                                losses=losses,
+                                pp_dynamic_shape_enabled=self.parallel_dims.pp_dynamic_shape_enabled,
+                                seq_len_multiple=self.seq_len_multiple,
+                            )
+                        loss = (
+                            torch.mean(torch.stack(losses)).to(self.device)
+                            if pp_last_stage
+                            else torch.tensor([-1.0], device=self.device)
                         )
                     else:
-                        # FWD + BWD if it is 1F1B-like scheduler
-                        self.pp_scheduler.step(
-                            position_ids=batch["position_ids"],
-                            target=targets,
-                            losses=losses,
-                            pp_dynamic_shape_enabled=self.parallel_dims.pp_dynamic_shape_enabled,
-                            seq_len_multiple=self.seq_len_multiple,
-                        )
-                    loss = (
-                        torch.mean(torch.stack(losses)).to(self.device)
-                        if pp_last_stage
-                        else torch.tensor([-1.0], device=self.device)
-                    )
-                else:
-                    logits = self.model(**batch)
+                        logits = self.model(**batch)
 
-                    # recover from ulysses if cp is enabled
-                    if self.parallel_dims.cp_enabled:
-                        batch["input_ids"] = input_ids_before_cp
-                        batch["position_ids"] = position_ids_before_cp
-                        if padding_mask_before_cp is not None:
-                            batch["padding_mask"] = padding_mask_before_cp
+                        # recover from ulysses if cp is enabled
+                        if self.parallel_dims.cp_enabled:
+                            batch["input_ids"] = input_ids_before_cp
+                            batch["position_ids"] = position_ids_before_cp
+                            if padding_mask_before_cp is not None:
+                                batch["padding_mask"] = padding_mask_before_cp
 
-                    logits = logits[:, :-1].contiguous()
-                    loss = self.loss_fn(
-                        logits.view(-1, logits.size(-1)),
-                        labels[:, 1:].contiguous().view(-1),
-                    )
-                    loss.backward()
-                loss = loss.detach()
+                        loss = self.loss_fn(logits, labels)
+                        loss = loss / len(mini_batches)
+                        loss.backward()
+                    acc_loss += loss.detach()
 
                 """
                 Compute the global grad norm on all parameters and then apply
@@ -564,11 +604,11 @@ class SFTTrainer(Trainer):
                     or self.parallel_dims.cp_enabled
                 ):
                     global_avg_loss, global_max_loss = (  # noqa: F841
-                        dist_util.dist_mean(loss, self.parallel_dims.mesh["dp_cp"]),
-                        dist_util.dist_max(loss, self.parallel_dims.mesh["dp_cp"]),
+                        dist_util.dist_mean(acc_loss, self.parallel_dims.mesh["dp_cp"]),
+                        dist_util.dist_max(acc_loss, self.parallel_dims.mesh["dp_cp"]),
                     )
                 else:
-                    global_avg_loss = global_max_loss = loss.item()  # noqa: F841
+                    global_avg_loss = global_max_loss = acc_loss.item()  # noqa: F841
 
                 if self.config.logging.logger:
                     if util.is_master_rank(self.parallel_dims, self.global_rank):
@@ -704,12 +744,14 @@ class SFTTrainer(Trainer):
 
     @property
     def pp_loss_fn(self):
-        def cross_entropy_loss(
-            output: torch.Tensor, target: torch.LongTensor
-        ) -> torch.Tensor:
-            """Common cross-entropy loss function for Transformer models training."""
-            return torch.nn.functional.cross_entropy(
-                output[:, :-1].flatten(0, 1).float(), target[:, 1:].flatten(0, 1)
-            )
-
-        return torch.compile(cross_entropy_loss)
+        # calculate the loss scaling factor
+        mini_batch_size = max(self.config.train.train_policy.mini_batch or 1, 1)
+        mini_batch_size = min(
+            mini_batch_size, self.config.train.train_batch_per_replica
+        )
+        loss_scaling_factor = (
+            mini_batch_size / self.config.train.train_batch_per_replica
+        )
+        return torch.compile(
+            partial(async_safe_ce, loss_scaling_factor=loss_scaling_factor)
+        )
