@@ -218,7 +218,6 @@ def _worker_loop(device_idx: int):
 
             deadline = time.monotonic() + task.timeout_ms / 1000.0
             # Poll async error status until success or timeout.
-            err = ncclResultEnum.ncclSuccess
             while time.monotonic() < deadline:
                 err = _nccl.ncclCommGetAsyncError(comm)
                 if err == ncclResultEnum.ncclSuccess:
@@ -235,9 +234,7 @@ def _worker_loop(device_idx: int):
 
             else:
                 # Enqueue timeout hit – abort communicator.
-                logger.error(
-                    f"NCCL: non-blocking enqueue timed out for task {task}, last Error {err}"
-                )
+                logger.error(f"NCCL: non-blocking enqueue timed out for task {task}")
                 _safe_abort(task.comm_idx, comm)
                 task.timed_out.set()
         except Exception as e:
